-- This work is licensed under a Creative Commons
-- Attribution-NonCommercial-ShareAlike 4.0 International License.
-- http://creativecommons.org/licenses/by-nc-sa/4.0

-- 2020 modifications:  Mark D. Aagaard, Nusa Zidaric

-- Mark D. Aagaard
-- Riham AlTawy
-- Guang Gong
-- Kalikinkar Mandal
-- Raghvendra Rohit
-- Marat Sattarov
-- Nusa Zidaric
-- http://uwaterloo.ca/communications-security-lab/lwc/wage


-- This is a human-readable summary of (and not a substitute for) the license. 
-- You are free to:

--     Share:  copy and redistribute the material in any medium or format
--     Adapt:  remix, transform, and build upon the material

--     The licensor cannot revoke these freedoms as long as you follow
--     the license terms.

-- Under the following terms:

--     Attribution - You must give appropriate credit, provide a link to
--     the license, and indicate if changes were made. You may do so in
--     any reasonable manner, but not in any way that suggests the
--     licensor endorses you or your use.

--     NonCommercial - You may not use the material for commercial
--     purposes.

--     ShareAlike - If you remix, transform, or build upon the material,
--     you must distribute your contributions under the same license as
--     the original.

--     No additional restrictions - You may not apply legal terms or
--     technological measures that legally restrict others from doing
--     anything the license permits.

------------------------------------------
-- control signals for DEC padding
------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.wage_pkg.all;
use work.ctrl_pkg.all;

entity ctrl_pad_to_ce is
  port
    ( i_padding 	: in  pad_ty
    ; ce_ctrl   	: out ce_ty
    ; pad_ctrl   	: out padxor_ty
   );
end entity;

architecture rtl of ctrl_pad_to_ce is

signal ce : ce_ty;
signal pad : padxor_ty;

begin

   ce_ctrl 	<= ce;
   pad_ctrl <= pad;

   ce      <= "11111111" when i_padding = "000" else
              "00000001" when i_padding = "001" else 
              "00000011" when i_padding = "010" else
              "00000111" when i_padding = "011" else
              "00001111" when i_padding = "100" else
              "00011111" when i_padding = "101" else
              "00111111" when i_padding = "110" else
              "01111111" ;              
    
   padxors: for j in 0 to 6  generate  
  		 pad(j) <= ce(j) xor  ce(j+1); 
 	end generate padxors;

end architecture;