-- This work is licensed under a Creative Commons
-- Attribution-NonCommercial-ShareAlike 4.0 International License.
-- http://creativecommons.org/licenses/by-nc-sa/4.0

-- 2020 modifications:  Mark D. Aagaard, Nusa Zidaric

-- Mark D. Aagaard
-- Riham AlTawy
-- Guang Gong
-- Kalikinkar Mandal
-- Raghvendra Rohit
-- Marat Sattarov
-- Nusa Zidaric
-- http://uwaterloo.ca/communications-security-lab/lwc/wage


-- This is a human-readable summary of (and not a substitute for) the license. 
-- You are free to:

--     Share:  copy and redistribute the material in any medium or format
--     Adapt:  remix, transform, and build upon the material

--     The licensor cannot revoke these freedoms as long as you follow
--     the license terms.

-- Under the following terms:

--     Attribution - You must give appropriate credit, provide a link to
--     the license, and indicate if changes were made. You may do so in
--     any reasonable manner, but not in any way that suggests the
--     licensor endorses you or your use.

--     NonCommercial - You may not use the material for commercial
--     purposes.

--     ShareAlike - If you remix, transform, or build upon the material,
--     you must distribute your contributions under the same license as
--     the original.

--     No additional restrictions - You may not apply legal terms or
--     technological measures that legally restrict others from doing
--     anything the license permits.


library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.wage_pkg.all;  
use work.ctrl_pkg.all;

entity wage is
  port
    ( clk,
      reset     	: in  std_logic
    ; pre_mode    : in  mode_ty
    ; pre_dom_sep : in  dom_sep_ty
    ; pre_padding : in  full_pad_ty
    ; pre_data    : in  word   
    ; pre_valid 	: in  std_logic
    
    -- post inputs
    ; post_ready 	: in  std_logic
    
    -- pre outputs 
    ; pre_ready   : out std_logic
     
    --post outputs
    ; post_data   : out word
    ; post_valid  : out std_logic
   );
end wage; 



architecture rtl of wage is
  signal ctl_control   	: dp_ctl_ty;
  signal ce_ctrl 			: ce_ty;
  signal pad_ctrl 		: padxor_ty;
  signal local_i_data,
         local_o_data  	: data_io;
  signal lfsr_c_r			: std_logic;
         
begin

  input: for i in 0 to 8 generate
    local_i_data(i) <= pre_data( i*gf_dim to (i+1)*gf_dim-1 );
  end generate;

  local_i_data(9) <= (0 => pre_data(63), others => '0');

  output: for i in 0 to 8 generate
    post_data( i*gf_dim to (i+1)*gf_dim-1 ) <= local_o_data(i);
  end generate;

  post_data(63) <= local_o_data(9)(0);
 


  u_dp 	: entity work.dp		port map ( clk, ce_ctrl, pad_ctrl, ctl_control, 
  													  pre_dom_sep, local_i_data, local_o_data  );
  													  
  u_ctl 	: entity work.ctrl 	port map ( clk, reset, pre_mode, pre_dom_sep, pre_valid, 
  													  post_ready, pre_padding, ctl_control, 
  													  ce_ctrl, pad_ctrl, post_valid, pre_ready   );

end architecture;  


     