-- This work is licensed under a Creative Commons
-- Attribution-NonCommercial-ShareAlike 4.0 International License.
-- http://creativecommons.org/licenses/by-nc-sa/4.0

-- 2020 modifications:  Mark D. Aagaard, Nusa Zidaric

-- Mark D. Aagaard
-- Riham AlTawy
-- Guang Gong
-- Kalikinkar Mandal
-- Raghvendra Rohit
-- Marat Sattarov
-- Nusa Zidaric
-- http://uwaterloo.ca/communications-security-lab/lwc/wage


-- This is a human-readable summary of (and not a substitute for) the license. 
-- You are free to:

--     Share:  copy and redistribute the material in any medium or format
--     Adapt:  remix, transform, and build upon the material

--     The licensor cannot revoke these freedoms as long as you follow
--     the license terms.

-- Under the following terms:

--     Attribution - You must give appropriate credit, provide a link to
--     the license, and indicate if changes were made. You may do so in
--     any reasonable manner, but not in any way that suggests the
--     licensor endorses you or your use.

--     NonCommercial - You may not use the material for commercial
--     purposes.

--     ShareAlike - If you remix, transform, or build upon the material,
--     you must distribute your contributions under the same license as
--     the original.

--     No additional restrictions - You may not apply legal terms or
--     technological measures that legally restrict others from doing
--     anything the license permits.


library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

package wage_pkg is

  -- for the wage_lfsr 

  constant gf_dim : integer := 7;
  constant N 		: integer := 37; -- number of stages
  constant D 		: integer := 10; -- number of data inputs/outputs
  subtype gf_elem is std_logic_vector( 0 to gf_dim-1 );
 
  type state_array is array ( N-1 downto 0 ) of gf_elem;
 
  type data_io is array ( D-1 downto 0 ) of gf_elem;
  type data_io_vector is array ( natural range <> ) of data_io;
  type gf_vector is array ( natural range <> ) of gf_elem;
  subtype word is std_logic_vector( 0 to 63 );

 -- constants for clean code  
  type 		ratestages_ty 			is array (N-1 	downto 0) of natural;
  type 		part_ratestages_ty 	is array (7 	downto 0) of natural;
  subtype 	stageidx_ty 			is std_logic_vector(N-1 downto 0);

  
   constant ratestages	 : ratestages_ty :=
 	 (36 => 9, 35 => 8, 34 => 7,  28 => 6, 27 => 5, 18 => 4, 16 => 3, 15 => 2, 9 => 1, 8 => 0
 	 , others => D+1 );    -- set others to out of range
 
   constant shiftplain 	 : stageidx_ty := 
    ( 33 => '1', 32 => '1', 31 => '1', 30 => '1', 26 => '1', 25 => '1', 24 => '1', 22 => '1', 21 => '1', 20 => '1', 19 => '1' 
    , 17 => '1', 14 => '1', 13 => '1', 12 => '1', 11 => '1',  7 => '1',  6 => '1',  5 => '1',  3 => '1',  2 => '1',  1 => '1'
    , others => '0'  );          
   constant absorbxor 	 : stageidx_ty := 
    ( 35 => '1', 34 => '1', 27 => '1', 18 => '1', 15 => '1',   8 => '1'
    , others => '0'  );  
   constant absorbxortag : stageidx_ty := 
    ( 28 => '1', 16 => '1',  9 => '1'
    , others => '0'  );  
   constant absorbplain  : stageidx_ty := 
    ( 35 => '1', 34 => '1', 28 => '1', 27 => '1', 16 => '1', 15 => '1',  9 => '1',  8 => '1'
    , others => '0'  );      
   constant replaceplain :  stageidx_ty :=  
    ( 35 => '1', 34 => '1', 28 => '1', 15 => '1',  9 => '1'
    , others => '0'  );      
   constant replaceload  :  stageidx_ty :=  
    ( 27 => '1', 18 => '1', 16 => '1', 8 => '1'
    , others => '0'  );     
   
 	constant part1 : part_ratestages_ty :=
    ( 6 => 35, 5 => 28, 4 => 27, 3 => 18, 2 => 16, 1 => 15
    , others => N+1);  -- set others to out of range
   constant part2 : part_ratestages_ty :=
    ( 6 =>  34, 5 => 27, 4 => 18, 3 => 16, 2 => 15, 1 => 9
    , others => N+1);  -- set others to out of range
    -- padbit index 
   constant pb 	: part_ratestages_ty :=
    ( 6 => 0,  5 => 6, 4 => 5, 3 => 4, 2 => 3, 1 => 2
 	 , others => D+1  );    -- set others to out of range     
  
 
  -- for the lfsr_c to produce the constants
  constant lfsr_c_sz:  integer := 7;
  type state_array_c is array ( lfsr_c_sz - 1 downto 0 ) of std_logic;

  -- for padding
  subtype pad_ty is std_logic_vector( 2 downto 0 );    		 
  subtype full_pad_ty is std_logic_vector( 3 downto 0 );    -- top lvl input

  subtype ce_ty is std_logic_vector( 7 downto 0 );   		 -- ce  ctrl for decryption
  subtype padxor_ty is std_logic_vector( 6 downto 0 );    -- pad ctrl for dectryption - pad detection

 end wage_pkg;




 



